# Copyright 2020 AccelByte Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import requests


# TODO: handling errorCode from backend
# TODO: move from returning response object to plain directory by .json() method
# TODO: do we wrap public method too?
class Wallet(object):
    """Accelbyte Wallet (Platform) Service

    lots of methods that interact with Wallet Service

    Args:
        namespace (str): namespace
        session (requests.Session): session that being used
        endpoint (str): API Endpoint URL
    
    Attributes:
        namespace (str)
        session (requests.Session)
    """
    WALLET_BODY_SOURCE = ['PURCHASE', 'IAP', 'PROMOTION',
                          'ACHIEVEMENT', 'REFERRAL_BONUS',
                          'REDEEM_CODE', 'REFUND', 'OTHER']

    def __init__(self, namespace, session, endpoint):
        self.namespace = namespace
        self.session = session
        self.base_url = endpoint
        self.admin_url = "{base}/platform/admin/namespaces/{namespace}".format(
            base=self.base_url, namespace=namespace)
        self.public_url = "{base}/platform/public/namespaces/{namespace}".format(
            base=self.base_url, namespace=namespace)
    
    def get_wallet(self, wallet_id):
        """Get a wallet by its id

        Args:
            wallet_id (int): wallet id
        
        Returns:
            requests.Response
        """
        url = "{base}/wallets/{wallet_id}".format(
            base=self.admin_url,
            wallet_id=wallet_id
        )
        return self.session.get(url)
    
    def get_user_wallet(self, user_id, wallet_id):
        """Get a wallet by its id within a user

        Args:
            user_id (int): user id
            wallet_id (int): wallet id
        
        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/wallets/{wallet_id}".format(
            base=self.admin_url,
            user_id=user_id,
            wallet_id=wallet_id
        )
        return self.session.get(url)
    
    def get_wallet_by_currency(self, user_id, currency):
        """Get a user wallet info by currency code

        Args:
            user_id (int): user id
            currency (str): country currency ISO code
        
        Returns:
            requests.Response
        """
        url = "{base}/wallets".format(base=self.admin_url)

        payload = {
            'userId': user_id,
            'currencyCode': currency
        }
        return self.session.get(url, params=payload)

    def credit(self, user_id, amount, currency,
               source="PURCHASE", reason="augment-default"):
        """Credit a user wallet by currency code
        
        If wallet not exists, it will create a new wallet.

        Args:
            user_id (int): user id
            amount (int): amount being credit
            currency (str): country currency ISO code
            source (str, optional): source
            reason (str, optional): reason

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/wallets/{currency}/credit".format(
            base=self.admin_url,
            user_id=user_id,
            currency=currency
        )

        # TODO: better handling for optional value of `source` and `reason`
        body = json.dumps({
            'amount': amount,
            'source': source,
            'reason': reason
        })
        return self.session.put(url, data=body)
    
    def debit(self, user_id, amount, wallet_id, reason="augment-default"):
        """Debit a user wallet

        Args:
            user_id (int): user id
            amount (int): amount being credit
            wallet_id (int): wallet id
            reason (str, optional): reason

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/wallets/{wallet_id}/debit".format(
            base=self.admin_url,
            user_id=user_id,
            wallet_id=wallet_id
        )

        body = json.dumps({
            'amount': amount,
            'reason': reason
        })
        return self.session.put(url, data=body)

    def pay(self, user_id, amount, currency):
        """Pay with user wallet by currency code

        Args:
            user_id (int): user id
            amount (int): amount being credit
            currency (str): country currency ISO code

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/wallets/{currency}/payment".format(
            base=self.admin_url,
            user_id=user_id,
            currency=currency
        )

        body = json.dumps({ 'amount': amount })
        return self.session.put(url, data=body)
    
    def enable(self, user_id, wallet_id):
        """Enable a user wallet

        Args:
            user_id (int): user id
            wallet_id (int): wallet id
        
        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/wallets/{wallet_id}/enable".format(
            base=self.admin_url,
            user_id=user_id,
            wallet_id=wallet_id
        )
        return self.session.put(url)

    def disable(self, user_id, wallet_id):
        """Disable a user wallet

        Args:
            user_id (int): user id
            wallet_id (int): wallet id
        
        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/wallets/{wallet_id}/disable".format(
            base=self.admin_url,
            user_id=user_id,
            wallet_id=wallet_id
        )
        return self.session.put(url)
    
    def get_transactions(self, user_id, wallet_id):
        """List user wallet transactions ordered by create time
        in descending manner

        Args:
            user_id (int): user id
            wallet_id (int): wallet id
        
        Returns:
            requests.Response        
        """
        url = "{base}/users/{user_id}/wallets/{wallet_id}/transactions".format(
            base=self.admin_url,
            user_id=user_id,
            wallet_id=wallet_id
        )
        return self.session.get(url)



# Copyright 2020 AccelByte Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import requests


class Session(object):
    """Session Manager

    An object to manage the current active session to call to the backend

    Args:
        client_id (str): IAM client id
        client_secret (str): IAM client secret
        endpoint (str): API Endpoint URL
    
    Attributes
        client_id (str)
        client_secret (str)
        session (requests.Session)
    """
    def __init__(self, client_id=None, client_secret=None, endpoint=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.base_url = endpoint
        self.grant_request_headers = self.__create_basic_auth_headers()
        self.session = requests.Session()

    def __create_basic_auth_headers(self):
        """Create basic auth headers

        Returns:
            headers with Basic Authorization
        """
        access_key = "{0}:{1}".format(self.client_id, self.client_secret)
        salted_key = base64.b64encode(
            access_key.encode("utf-8")).decode("utf-8")
        basic_auth = "Basic {0}".format(salted_key)
        headers = {'Authorization': basic_auth}
        return headers
    
    def init_client_credentials_grant(self):
        """Get grant by client credential

        Returns:
            requests.Session
        """
        token_url = "{0}/iam/oauth/token".format(self.base_url)
        grant_request_body = {
            'grant_type': 'client_credentials',
            'username': '',
            'password': '',
            'refresh_token': '',
            'code': '',
            'redirect_url': '',
            'namespace': ''
        }

        response = requests.post(token_url,
            data=grant_request_body, headers=self.grant_request_headers)
        token = response.json()

        self.session.headers = {
            'content-type': 'application/json',
            'Accept': 'application/json',
            'Authorization': "Bearer {0}".format(token['access_token'])
        }
        return self.session

    def init_password_grant(self, username, password):
        """Get grant by password

        Args:
            username (str)
            password (str)

        Returns:
            requests.Session
        """
        token_url = "{0}/iam/oauth/token".format(self.base_url)
        grant_request_body = {
            'grant_type': 'password',
            'username': username,
            'password': password,
            'refresh_token': '',
            'code': '',
            'redirect_url': '',
            'namespace': ''
        }

        response = requests.post(token_url,
            data=grant_request_body, headers=self.grant_request_headers)
        token = response.json()

        self.session.headers = {
            'content-type': 'application/json',
            'Accept': 'application/json',
            'Authorization': "Bearer {0}".format(token['access_token'])
        }
        return self.session



# Copyright 2020 AccelByte Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import requests

from .session import Session
from .wallet import Wallet
from .statistic import Statistic


class Justice(object):
    """Justice Python SDK for Augment

    This is singleton class to interact with other justice service objects

    Args:
        namespace (str): Namespace
        endpoint (str): API endpoint URL

    Attributes:
        session (requests.Session): Python `requests.Session` object that is
            currently active.
        wallet (justice.Wallet): Wallet object to interact with Justice Wallet
            Service.
        statistic (justice.Statistic): Statistic object to interact with justice
            Statistic Service.
    """
    DEFAULT_ENDPOINT = "https://demo.accelbyte.io"

    def __init__(self, namespace, endpoint=DEFAULT_ENDPOINT):
        if not self.valid_url(endpoint):
            message = "{} is not a valid API endpoint URL".format(endpoint)
            raise ValueError(message)

        # create session
        client_id = os.environ['IAM_CLIENT_ID']
        client_secret = os.environ['IAM_CLIENT_SECRET']
        sess = Session(client_id, client_secret, endpoint)

        # init session with client credentials to get grant token
        self.session = sess.init_client_credentials_grant()

        # initialise accelbyte services objects
        self.__init_services_object(namespace, endpoint)

    def __init_services_object(self, namespace, endpoint):
        """Private method to expose available service objects"""
        self.wallet = Wallet(namespace, self.session, endpoint)
        self.statistic = Statistic(namespace, self.session, endpoint)

    def valid_url(self, url):
        """Check if given string is a valid URL scheme.

        Args:
            url (str): string that contains URL to be checked.

        Returns:
            re.Match object, None otherwise.
        """
        pattern = re.compile(
            r"^(?:http(s)?:\/\/)?[\w.-]+(?:\.[\w\.-]+)+[\w\-\._~:/?#[\]@!\$&'\(\)\*\+,;=.]+$")
        return pattern.match(url)



# Copyright 2020 AccelByte Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import requests


# NOTE: we only wrap admin methods for now
class Statistic(object):
    """Accelbyte Statistic Service

    This object provides wrapped methods to do CRUD operations to
    Justice Statistic Service.

    Args:
        namespace (str): Namespace
        session (requests.Session): Session that being used.
        endpoint (str): API Endpoint URL

    Attributes:
        namespace (str)
        session (requests.Session)
    """

    def __init__(self, namespace, session, endpoint):
        self.namespace = namespace
        self.session = session
        self.base_url = endpoint
        self.admin_url = "{base}/statistic/v1/admin/namespaces/{namespace}".format(
            base=self.base_url, namespace=namespace)
        self.public_url = "{base}/statistic/v1/public/namespaces/{namespace}".format(
            base=self.base_url, namespace=namespace)

    def get_global_stats(self):
        """List all global statItems by given namespace

        Returns:
            requests.Response
        """
        url = "{base}/globalstatitems".format(base=self.admin_url)
        return self.session.get(url)

    def search_statcode(self, keyword):
        """Query all stats by given namespace and stat code keyword

        Args:
            keyword (str): Stat code keyword.

        Returns:
            requests.Response
        """
        url = "{base}/stats/search".format(base=self.admin_url)
        payload = {'keyword': keyword}
        return self.session.get(url, params=payload)

    def get_stat_config(self, stat_code):
        """Get stat item configuration by its statCode

        Args:
            stat_code (str): statCode

        Returns:
            requests.Response
        """
        url = "{base}/stats/{stat_code}".format(
            base=self.admin_url,
            stat_code=stat_code
        )
        return self.session.get(url)

    def create_stat_config(self, config):
        """Create stat item configuration

        config example:
        ```python
        config = {
            "defaultValue": 0,
            "description": "string",
            "incrementOnly": True,
            "maximum": 0,
            "minimum": 0,
            "name": "string",
            "setAsGlobal": True,
            "setBy": "SERVER",
            "statCode": "string",
            "tags": [
                "string"
            ]
        }
        ```

        Args:
            stat_code (str): statCode
            config (dict): Stat configuration dict.

        Returns:
            requests.Response
        """
        url = "{base}/stats".format(base=self.admin_url)
        body = json.dumps(config)
        return self.session.post(url, data=body)

    def update_stat_config(self, stat_code, description=None, name=None, tags=None):
        """Update stat item configuration by its statCode

        Args:
            stat_code (str): statCode
            description (str, optional): Description to be updated. Defaults to None.
            name (str, optional): Name to be updated. Defaults to None.
            tags (str|list, optional): Tags to be updated. Defaults to None.

        Returns:
            requests.Response
        """
        url = "{base}/stats/{stat_code}".format(
            base=self.admin_url,
            stat_code=stat_code
        )
        payload = {}

        if description:
            payload['description'] = description
        if name:
            payload['name'] = name
        if tags and len(tags):
            if isinstance(tags, str):
                payload['tags'] = [tags]
            payload['tags'] = tags

        body = json.dumps(payload)
        return self.session.patch(url, data=body)

    def delete_stat_config(self, stat_code):
        """Delete a stat item configuration by its statCode

        Args:
            stat_code (str): Stat code that needs to be deleted.

        Returns:
            requests.Response
        """
        url = "{base}/stats/{stat_code}".format(
            base=self.admin_url,
            stat_code=stat_code
        )
        return self.session.delete(url)

    def get_user_stats(self, user_id):
        """Get the statistic for a user based on its id

        This will list all active statitems for that user

        Args:
            user_id (str): User id.

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/statitems".format(
            base=self.admin_url,
            user_id=user_id
        )
        return self.session.get(url)

    def create_user_statitem(self, user_id, stat_code):
        """Create a stat item for one user

        Args:
            user_id (str): User id.
            stat_code (str): statCode

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/stats/{stat_code}/statitems".format(
            base=self.admin_url,
            user_id=user_id,
            stat_code=stat_code
        )
        return self.session.post(url)

    def update_user_statitem_value(self, user_id, stat_code, value):
        """Update a stat item for a user

        Args:
            user_id (str): User id
            stat_code (str): statCode
            value (int): Stat item value to be updated.

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/stats/{stat_code}/statitems/value".format(
            base=self.admin_url,
            user_id=user_id,
            stat_code=stat_code
        )

        body = json.dumps({'inc': value})
        return self.session.patch(url, data=body)

    def delete_user_statitem(self, user_id, stat_code):
        """Delete a stat item for a user

        Args:
            user_id (str): User id
            stat_code (str): statCode

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/stats/{stat_code}/statitems".format(
            base=self.admin_url,
            user_id=user_id,
            stat_code=stat_code
        )
        return self.session.delete(url)

    def put_update_multiple_bulk(self, bulk):
        """Update multiple users' stats via PUT

        Scheme:
        ```python
        bulk = [
            {
                "inc": 0,
                "statCode": "string",
                "userId": "string"
            }
        ]
        ```

        Args:
            bulk (list of scheme): Bulk payloads.

        Returns:
            requests.Response
        """
        url = "{base}/statitems/value/bulk".format(base=self.admin_url)
        body = json.dumps(bulk)
        return self.session.put(url, data=body)

    def patch_update_multiple_bulk(self, bulk):
        """Update multiple users' stats via PATCH

        Scheme:
        ```python
        bulk = [
            {
                "inc": 0,
                "statCode": "string",
                "userId": "string"
            }
        ]
        ```

        Args:
            bulk (list of scheme): Bulk payloads.

        Returns:
            requests.Response
        """
        url = "{base}/statitems/value/bulk".format(base=self.admin_url)
        body = json.dumps(bulk)
        return self.session.patch(url, data=body)

    def create_single_bulk(self, user_id, bulk):
        """Bulk create stat items for one user

        The endpoint expect the payload to be like this:
        ```json
        [
            { "statCode": "string" }
        ]
        ```

        Args:
            bulk (list): list of statcode to be created

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/statitems/bulk".format(
            base=self.admin_url,
            user_id=user_id
        )

        payload = [{"statCode": item} for item in bulk]
        body = json.dumps(payload)
        return self.session.post(url, data=body)

    def put_update_single_bulk(self, user_id, bulk):
        """Bulk update one user stats via PUT

        Scheme:
        ```python
        bulk = [
            {
                "inc": 0,
                "statCode": "string"
            }
        ]
        ```

        Args:
            bulk (list of scheme): Bulk payloads.

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/statitems/value/bulk".format(
            base=self.admin_url,
            user_id=user_id
        )
        body = json.dumps(bulk)
        return self.session.put(url, data=body)

    def patch_update_single_bulk(self, user_id, bulk):
        """Bulk update one user stats via PATCH

        Scheme:
        ```python
        bulk = [
            {
                "inc": 0,
                "statCode": "string"
            }
        ]
        ```

        Args:
            bulk (list of scheme): Bulk payloads.

        Returns:
            requests.Response
        """
        url = "{base}/users/{user_id}/statitems/value/bulk".format(
            base=self.admin_url,
            user_id=user_id
        )
        body = json.dumps(bulk)
        return self.session.patch(url, data=body)



# Copyright 2020 AccelByte Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import urllib.parse
import os

from pymongo import MongoClient


class BuiltInDB(object):
    """Python SDK for Augment Built-In Database

    This is singleton class to interact with Augment built-in database

    Args:
        namespace (str): Namespace
        endpoint (str): API endpoint URL

    Attributes:
        database: Python object tat interacts with built-in database
    """
    DEFAULT_MONGODB = "abcluster-accelbyte-prod-docdb-cluster-0.cfunpavjqvdi.us-west-2.docdb.amazonaws.com:27017/" + \
                      "?replicaSet=rs0&readPreference=secondaryPreferred"

    def __init__(self, endpoint=DEFAULT_MONGODB):
        # create mongoDB client session
        username = urllib.parse.quote_plus(os.environ['BUILTIN_DB_USER_NAME'])
        password = urllib.parse.quote_plus(os.environ['BUILTIN_DB_USER_PASSWORD'])
        db_name = urllib.parse.quote_plus(os.environ['BUILTIN_DB_NAME'])
        try:
            client = MongoClient('mongodb://%s:%s@%s' % (username, password, endpoint))

            # assign the database name
            self.database = client[db_name]
        except Exception as exception:
            raise exception

    def insert(self, collection_name, data):
        """Insert a dictionary of data to the built-in database

        Args:
            collection_name (string): name of the collection to insert the data into
            data (dict): data to insert

        Returns:
            data (dict)
        """
        try:
            self.database[collection_name].insert_one(data)
        except Exception as exception:
            raise exception

        return data

    def get(self, collection_name, get_filter={}):
        """Get data in a collection

        Args:
            collection_name (string): name of the collection to insert the data into
            get_filter (dict): filter dict to get data

        Returns:
            data (dict)
        """
        try:
            result_data = self.database[collection_name].find_one(get_filter)
        except Exception as exception:
            raise exception

        return result_data

    def update(self, collection_name, update_filter, new_data):
        """Update data in a collection by replacing with the new data

        Args:
            collection_name (string): name of the collection to insert the data into
            update_filter (dict): filter dict to get the data to be updated
            new_data (dict): new data to replace the existing data

        Returns:
            new_data (dict)
        """
        try:
            update_result = self.database[collection_name].replace_one(update_filter, new_data)
            if update_result.matched_count == 0:
                raise Exception("data to update not found")
        except Exception as exception:
            raise exception

        return new_data

    def delete(self, collection_name, delete_filter):
        """Update data in a collection by replacing with the new data

        Args:
            collection_name (string): name of the collection to insert the data into
            delete_filter (dict): filter dict to get the data to be deleted
        """
        try:
            self.database[collection_name].delete_one(delete_filter)
        except Exception as exception:
            raise exception




# Copyright (c) 2020 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

import json

from setuptools import setup, find_packages


# Package dependencies
requires = [
    'requests',
    'pymongo'
]
test_requires = [
    'tox'
]

license_string = "Apache 2.0"

with open('README.md', encoding='utf-8') as f:
    readme = f.read()

with open('version.json') as f:
    info = json.load(f)

setup(
    name='justice-augment-python-sdk',
    version=info['version'],
    description='Python SDK for Justice Augment',
    long_description=readme,
    long_description_content_type='text/markdown; variant=GFM',
    author='Bahrunnur',
    author_email='bahrunnur@accelbyte.io',
    url='https://bitbucket.org/accelbyte/justice-augment-python-sdk',
    license=license_string,
    packages=['justice', 'datastore'],
    install_requires=requires,
    tests_require=test_requires,
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
)
